﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using projectDoubleDeckerBus.Drawings;
using projectDouble_Decker_Bus.MovementStrategy;
using projectDoubleDeckerBus.Generics;

namespace projectDoubleDeckerBus
{
    internal class BusesGenericCollection<T, U>
    where T : DrawningBus
    where U : IMoveableObject
    {
        /// <summary>
        /// Ширина окна прорисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна прорисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 172;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 112;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetGeneric<T> _collection;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        public BusesGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="collect"></param>
        /// <param name="obj"></param>
        /// <returns></returns>
        /// 
        public static int? operator +(BusesGenericCollection<T, U> collect, T?
        obj)
        {
            if (obj == null)
            {
                return -1;
            }
            return collect?._collection.Insert(obj);
        }
        public static bool operator -(BusesGenericCollection<T, U> collect, int
        pos)
        {
            T? obj = collect._collection.Get(pos);
            if (obj == null)
            {
                return false;
            }
            return collect._collection.Remove(pos);
        }

        public U? GetU(int pos)
        {
            return (U?)_collection.Get(pos)?.GetMoveableObject;
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowCars()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {//линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawObjects(Graphics g)
        {
            for (int i = 0; i < _collection.Count; i++)
            {
                int width = _pictureWidth / _placeSizeWidth;
                int height = _pictureHeight / _placeSizeHeight;
                for (int j = 0; j < _collection.Count; j++)
                {
                    DrawningBus? bus = _collection.Get(j);
                    if (bus == null)
                        continue;
                    bus.SetPosition((width - j % width - 1) * _placeSizeWidth, j / width * _placeSizeHeight);
                    bus.DrawTransport(g);
                }
            }
        }
    }
}
