﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using projectDoubleDeckerBus;
using System.Drawing;
using projectDoubleDeckerBus.Entity;
using projectDouble_Decker_Bus.MovementStrategy;

namespace projectDoubleDeckerBus.Drawings
{
    public class DrawningBus
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBus? EntityBus { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// 
        /// Левая координата прорисовки автобуса
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автобуса
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки автобуса
        /// </summary>
        protected readonly int _busWidth = 172;
        /// <summary>
        /// Высота прорисовки автобуса
        /// </summary>
        protected readonly int _busHeight = 112;
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _busWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _busHeight;

        public IMoveableObject GetMoveableObject => new
        DrawningObjectBus(this);
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена,
        /// 
        public bool CanMove(Direction direction)
        {
            if (EntityBus == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                Direction.Left => _startPosX - EntityBus.Step > 0,
                //вверх
                Direction.Up => _startPosY - EntityBus.Step > 0,
                //Down
                Direction.Down => _startPosY + EntityBus.Step < _pictureHeight,
                //Right
                Direction.Right => _startPosX + EntityBus.Step < _pictureWidth
            };
        }

        public void MoveTransport(Direction direction)
        {
            if (EntityBus == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case Direction.Left:
                    if (_startPosX - EntityBus.Step > 0)
                    {
                        _startPosX -= (int)EntityBus.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - EntityBus.Step > 0)
                    {
                        _startPosY -= (int)EntityBus.Step;
                    }
                    break;
                // вправо
                case Direction.Right:
                    if (_startPosX + _busWidth + EntityBus.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityBus.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _busHeight + EntityBus.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityBus.Step;
                    }
                    break;
            }
        }
        public DrawningBus(int speed, double weight, Color bodyColor, int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBus = new EntityBus(speed, weight, bodyColor);

            /// <summary>
            /// Проверка на вместимость обьекта в рамки сцены
            /// </summary>
            if ((_busWidth >= _pictureWidth) || (_busHeight >= _pictureHeight))
            {
                Console.WriteLine("проверка не пройдена, нельзя создать объект в этих размерах");
                if (_busWidth >= _pictureWidth)
                {
                    _busWidth = _pictureWidth - _busWidth;
                }
                if (_busHeight >= _pictureWidth)
                {
                    _busHeight = _pictureWidth - _busHeight;
                }
            }
            else
                Console.WriteLine("объект создан");
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="busWidth">Ширина прорисовки автомобиля</param>
        /// <param name="busHeight">Высота прорисовки автомобиля</param>
        protected DrawningBus(int speed, double weight, Color bodyColor, int
        width, int height, int busWidth, int busHeight)
        {
            if (busWidth >= width || busHeight >= height)
            {
                Console.WriteLine("Проверка не пройдена, нельзя создать объект в этих размерах");
                if (busWidth >= width)
                {
                    busWidth = width - busWidth;
                }
                if (busHeight >= height)
                {
                    busHeight = height - busHeight;
                }
            }
            else
                Console.WriteLine("Объект создан");

            _pictureWidth = width;
            _pictureHeight = height;
            _busWidth = busWidth;
            _busHeight = busHeight;
            EntityBus = new EntityBus(speed, weight, bodyColor);
        }

        public void SetPosition(int x, int y)
        {
            if (((x >= 0) && (x + _busWidth > _pictureWidth)) || ((y >= 0) && (y + _busHeight > _pictureHeight)))
            {
                _startPosX = _pictureWidth - _busWidth;
                _startPosY = _pictureHeight - _busHeight;
            }

            else
            {
                _startPosX = x;
                _startPosY = y;
            }
        }

        public void MoveBus(Direction direction)
        {
            if (EntityBus == null)
            {
                return;
            }

            switch (direction)
            {
                //влево
                case Direction.Left:
                    if (_startPosX - EntityBus.Step > 0)
                    {
                        _startPosX -= (int)EntityBus.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - EntityBus.Step > 0)
                    {
                        _startPosY -= (int)EntityBus.Step;
                    }
                    break;
                // вправо
                case Direction.Right:
                    if (_startPosX + EntityBus.Step + _busWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityBus.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + EntityBus.Step + _busHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityBus.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBus == null)
            {
                return;
            }

            int windowSize = 15;
            Brush brBlack = new SolidBrush(Color.Black);
            Brush brRed = new SolidBrush(Color.Red);

            Pen pen = new(Color.Black);
           
            Brush br = new SolidBrush(EntityBus?.BodyColor ?? Color.Black);
            g.FillRectangle(br, _startPosX + 40, _startPosY + 34, 130, 52);

            g.DrawEllipse(pen, _startPosX + 46, _startPosY + 50, windowSize, windowSize);
            g.DrawEllipse(pen, _startPosX + 62, _startPosY + 50, windowSize, windowSize);
            g.DrawEllipse(pen, _startPosX + 100, _startPosY + 50, windowSize, windowSize);

            g.DrawEllipse(pen, _startPosX + 115, _startPosY + 50, windowSize, windowSize);
            g.DrawEllipse(pen, _startPosX + 130, _startPosY + 50, windowSize, windowSize);
            g.DrawEllipse(pen, _startPosX + 145, _startPosY + 50, windowSize, windowSize);

           
            g.FillRectangle(brBlack, _startPosX + 80, _startPosY + 46, 15, 40);
            g.FillEllipse(brBlack, _startPosX + 50, _startPosY + 75, 20, 20);
            g.FillEllipse(brBlack, _startPosX + 140, _startPosY + 75, 20, 20);

            Brush brBlue = new SolidBrush(Color.LightBlue);
            g.FillEllipse(brBlue, _startPosX + 46, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 62, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 100, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 115, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 130, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 145, _startPosY + 50, windowSize, windowSize);

        }
    }
}
