using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using projectDoubleDeckerBus.Drawings;
using projectDoubleDeckerBus.Entity;
using projectDouble_Decker_Bus.MovementStrategy;

namespace projectDoubleDeckerBus
{
    public partial class FormDoubleDeckerBus : Form
    {
        private DrawningBus? _drawningBus;

        private AbstractStrategy? _abstractStrategy;

        private AbstractStrategy? _strategy;
        /// <summary>
        ///  
        /// </summary>
        public DrawningBus? SelectedBus { get; private set; }

        public FormDoubleDeckerBus()
        {
            InitializeComponent();
            _strategy = null;
            SelectedBus = null;
        }

        private void Draw()
        {
            if (_drawningBus == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxDoubleDeckerBus.Width,
            pictureBoxDoubleDeckerBus.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningBus.DrawTransport(gr);
            pictureBoxDoubleDeckerBus.Image = bmp;
        }
        private void buttonCreateDoubleDeckerBus_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color mainColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                mainColor = dialog.Color;
            }
            Color addColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                addColor = dialog.Color;
            }
            _drawningBus = new DrawningDoubleDeckerBus(random.Next(100, 300),
             random.Next(1000, 3000), mainColor,
             addColor, Convert.ToBoolean(random.Next(0, 2)),
             Convert.ToBoolean(random.Next(0, 2)),
             pictureBoxDoubleDeckerBus.Width, pictureBoxDoubleDeckerBus.Height);
            _drawningBus.SetPosition(random.Next(10, 100), random.Next(10,
            100));

            Draw();
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningBus == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningBus.MoveTransport(Direction.Up);
                    break;
                case "buttonDown":
                    _drawningBus.MoveTransport(Direction.Down);
                    break;
                case "buttonLeft":
                    _drawningBus.MoveTransport(Direction.Left);
                    break;
                case "buttonRight":
                    _drawningBus.MoveTransport(Direction.Right);
                    break;
            }
            Draw();
        }

        private void buttonCreateBus_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningBus = new DrawningBus(random.Next(100, 300),
            random.Next(1000, 3000), color,
            pictureBoxDoubleDeckerBus.Width, pictureBoxDoubleDeckerBus.Height);
            _drawningBus.SetPosition(random.Next(10, 100), random.Next(10,
            100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningBus == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawningObjectBus(_drawningBus), pictureBoxDoubleDeckerBus.Width,
                pictureBoxDoubleDeckerBus.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }

        }

        private void buttonAddBus_Click(object sender, EventArgs e)
        {
            SelectedBus = _drawningBus;
            DialogResult = DialogResult.OK;
        }
    }
}